      SUBROUTINE DRAG (RADIUS,AREA,SIGMA,TEMP,PRES,DENS,VEL,REY)
C
C        CALCULATE PRESSURE DROP DUE TO DRAG ACROSS SCREENS
C
C        INPUT:
C
C        RADIUS   REAL       RADIUS OF INTEREST
C        AREA     REAL       AREA OF SCREEN
C        SIGMA    REAL       CONSTANT EQUAL TO 3*SQRT((PI*M)/(8*K))
C                              WHERE M=MOLECULE MASS, K=BOLTZMANN CONST
C        PRES     REAL       FLOW PRESSURE
C        DENS     REAL       FLOW DENSITY
C        TEMP     REAL       FLOW TEMPERATURE
C        VEL      REAL       FLOW VELOCITY
C
C        OUTPUT:
C
C        PRES     REAL       ADJUSTED FLOW PRESSURE
C        REY      REAL       REYNOLD'S NUMBER
C
C        WE ASSUME DRAG IS 0.5*CD*AREA*VEL**2
C
      IMPLICIT REAL (A-H,O-Z)
C
C        CALCULATE REYNOLD'S NUMBER
C
      REY=SIGMA*VEL/(DENS*SQRT(TEMP))
C
C        GET DRAG COEFFICIENT
C
      COEFF=CD(REY)
C
C        CALCULATE DRAG
C
      PRES=PRES+0.5*COEFF*DENS*AREA*VEL*VEL
      END
